Program BarGraph;
Uses crt;
Const
  maxx = 25;              {Max classes to graph}
  maxy= 20;               {Max y intervals}
  hchar = #196;           {Horizontal line char}
  vchar = #179;           {Vertical line char}
  width = 60;             {Graph width in chars}
  height = 20;            {Graph height in lines}
  vstart = 22;            {Vertical origin}
  hstart = 10;            {Horizontal origin}
  lablgth = 9;            {Max length of y values}
  dplaces = 1;            {Decimal places for y values}
  titlelen = 10;          {Max length of a title string}
  barchar = #219#219#219; {Bar graph drawing char}
  msglen = 30;            {Max length of error message}
Type
  posarray = array[1..maxx] of byte;
  yarray = array[1..maxx] of real;
  titlearray = array[1..maxx] of string[titlelen];
  msgstr = string[msglen];
Var
  xpos : posarray;    {Stores title positions}
  y : yarray;         {Stores y values}
  title : titlearray; {Stores class titles}
  errcode, nrx : integer;
  hspace, vspace : real;

Procedure terminate(msg : msgstr); {Halts execution if error
                                    condition arises}
Begin
  writeln(msg);
  halt;
end; {terminate}

Procedure get_parms; {Sets up the command line parameters}
Var i : integer;
Begin
  if (paramcount = 0) or (paramcount mod 2 <> 0) then
    terminate('Invalid number of parameters')
  else
    begin
      nrx:=paramcount div 2;
      for i:=1 to nrx do
        title[i]:=paramstr(i);
      for i:=1 to nrx do
        begin
          val(paramstr(nrx+i),y[i],errcode);
          if errcode <> 0 then
            terminate('Invalid number');
        end;
    end;
end; {get_params}

Procedure draw_x_axis; {Displays x-axis and class titles}
Var titlelgth, i : integer;

Procedure get_title_length; {Gets maximum length of class titles}
Var i : integer;
Begin
  titlelgth:=0;
  for i:=1 to nrx do
    if length(title[i]) > titlelgth then
      titlelgth:=length(title[i]);
end; {get_title_length}

Begin
  get_title_length;
  hspace:=width div nrx;
  if titlelgth+1 > hspace then
    terminate('Insufficient room for titles')
  else
    begin
      gotoxy(hstart-1,vstart);
      for i:=0 to width do
        write(hchar);
      for i:=1 to nrx do
        begin
          xpos[i]:=hstart+trunc(hspace-titlelgth) div 2
                   +(i-1)*trunc(hspace);
          gotoxy(xpos[i],vstart+1);
          write(title[i]);
        end;
    end;
end; {draw_x_axis}

Procedure draw_y_axis; {Displays y-axis and values}
Var i : integer;
    maxy : real;

Procedure get_max_y; {Finds the maximum y value}
Var i : integer;
Begin
  maxy:=0;
  for i:=1 to nrx do
    begin
      if y[i] < 0 then
        terminate('Negative values not permitted')
      else
        if y[i] > maxy then
          maxy:=y[i];
    end;
  maxy:=trunc(maxy)+1;
end; {get_max_y}

Begin
  get_max_y;
  vspace:=height div trunc(maxy);
  gotoxy(hstart,vstart);
  for i:=1 to height do
    begin
      write(vchar);
      gotoxy(wherex-1,wherey-1);
    end;
  gotoxy(1,vstart-trunc(maxy*vspace));
  write(maxy:lablgth:1);
  gotoxy(1,vstart-trunc(maxy/2*vspace));
  write(maxy/2:lablgth:1)
end; {draw_y_axis}

Procedure draw_graph; {Draws the bar graph}
Var i, j : integer;
Begin
  for i:=1 to nrx do
    begin
      gotoxy(xpos[i],vstart-trunc(y[i]*vspace));
      while wherey < vstart do
        begin
          write(barchar);
          gotoxy(wherex-3,wherey+1);
        end;
    end;
  gotoxy(1,25);
end; {draw_graph}

Begin {BarGraph}
  clrscr;
  get_parms;
  textcolor(green);
  draw_x_axis;
  draw_y_axis;
  textcolor(blue);
  draw_graph;
end.